# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import file


@cli.command("file-list", help="List all files.")
@click.option("--job-id", required=True)
@click.pass_obj
def list(context, job_id):
    """list(context)

    List all files.

    >>> dcictl file-list [OPTIONS]

    :param string job_id: ID of the job [required]
    """
    result = file.list(context, where='job_id:' + job_id)
    utils.format_output(result, context.format)


@cli.command("file-show", help="Show a file.")
@click.argument("id", required=True)
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a file.

    >>> dcictl file-show [OPTIONS]

    :param string id: ID of the file to show [required]
    """
    content = file.content(context, id=id)
    click.echo(content.text)


@cli.command("file-delete", help="Delete a file.")
@click.argument("id")
@click.pass_obj
def delete(context, id):
    """delete(context, id)

    Delete a file.

    >>> dcictl file-delete [OPTIONS]

    :param string id: ID of the file to delete [required]
    """
    result = file.delete(context, id=id)

    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'File deleted.'})
    else:
        utils.format_output(result, context.format)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import file as dci_file
from dciclient.v1.api import job


@cli.command("job-list", help="List all jobs.")
@click.option("--sort", default="-created_at")
@click.option("--limit", help="Number of jobs to show up.",
              required=False, default=10)
@click.option("--where", help="An optional filter criteria.",
              required=False)
@click.pass_obj
def list(context, sort, limit, where):
    """list(context)

    List all jobs.

    >>> dcictl job-list

    :param string sort: Field to apply sort
    :param integer limit: Max number of rows to return
    :param string where: An optional filter criteria
    """
    result = job.list(
        context, sort=sort, limit=limit, where=where,
        embed='jobdefinition,remoteci,team')
    headers = ['id', 'status', 'recheck',
               'jobdefinition/name', 'remoteci/name',
               'team/name', 'etag', 'created_at', 'updated_at']

    utils.format_output(result, context.format, headers)


@cli.command("job-show", help="Show a job.")
@click.argument('id', required=False)
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a job.

    >>> dcictl job-show [OPTIONS]

    :param string id: ID of the job to show [required]
    """
    result = job.get(context, id=id)
    utils.format_output(result, context.format)


@cli.command("job-delete", help="Delete a job.")
@click.argument("id")
@click.option("--etag", required=True)
@click.pass_obj
def delete(context, id, etag):
    """delete(context, id, etag)

    Delete a job.

    >>> dcictl job-delete [OPTIONS]

    :param string id: ID of the job to delete [required]
    :param string etag: Entity tag of the job resource [required]
    """
    result = job.delete(context, id=id, etag=etag)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Job deleted.'})
    else:
        utils.format_output(result, context.format)


@cli.command("job-recheck", help="Recheck a job.")
@click.argument("id")
@click.pass_obj
def recheck(context, id):
    """recheck(context, id)

    Recheck a job.

    >>> dcictl job-recheck [OPTIONS]

    :param string id: ID of the job to recheck [required]
    """
    result = job.recheck(context, id=id)
    utils.format_output(result, context.format)


@cli.command("job-results", help="List all job results.")
@click.argument("id")
@click.option("--sort", default="-created_at")
@click.option("--limit", default=50)
@click.pass_obj
def list_results(context, id, sort, limit):
    """list_result(context, id)

    List all job results.

    >>> dcictl job-results [OPTIONS]

    :param string id: ID of the job to recheck [required]
    :param string sort: Field to apply sort
    :param integer limit: Max number of rows to return
    """

    headers = ['filename', 'name', 'total', 'success', 'failures', 'errors',
               'skips', 'time']
    result = job.list_results(context, id=id, sort=sort, limit=limit)
    utils.format_output(result, context.format, headers)


@cli.command("job-attach-issue", help="Attach an issue to a job.")
@click.argument("id")
@click.option("--url", required=True)
@click.pass_obj
def attach_issue(context, id, url):
    """attach_issue(context, id, url)

    Attach an issue to a job.

    >>> dcictl job-attach-issue [OPTIONS]

    :param string id: ID of the job to attach the issue to [required]
    :param string url: URL of the issue to attach to the job [required]
    """

    result = job.attach_issue(context, id=id, url=url)
    utils.format_output(result, context.format)


@cli.command("job-unattach-issue", help="Unattach an issue from a job.")
@click.argument("id")
@click.option("--issue_id", required=True)
@click.pass_obj
def unattach_issue(context, id, issue_id):
    """unattach_issue(context, id, issue_id)

    Unattach an issue from a job.

    >>> dcictl job-unattach-issue [OPTIONS]

    :param string id: ID of the job to attach the issue to [required]
    :param string issue_id: ID of the issue to unattach from the job [required]
    """

    result = job.unattach_issue(context, id=id, issue_id=issue_id)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Issue unattached.'})
    else:
        utils.format_output(result, context.format)


@cli.command("job-list-issue", help="List all job attached issues.")
@click.argument("id")
@click.option("--sort", default="-created_at")
@click.option("--limit", default=50)
@click.pass_obj
def list_issues(context, id, sort, limit):
    """list_issues(context, id)

    List all job attached issues.

    >>> dcictl job-list-issue [OPTIONS]

    :param string id: ID of the job to retrieve issues from [required]
    :param string sort: Field to apply sort
    :param integer limit: Max number of rows to return
    """

    result = job.list_issues(context, id=id, sort=sort, limit=limit)
    headers = ['id', 'status', 'product', 'component', 'title', 'url']
    utils.format_output(result, context.format, headers)


@cli.command("job-output", help="Show the job output.")
@click.argument('id')
@click.pass_obj
def output(context, id):
    """output(context, id)

    Show a job output.

    >>> dcictl job-output [OPTIONS]

    :param string id: ID of the job to show [required]
    """

    colors = {
        'pre-run': '\x1b[6;30;44m',
        'running': '\x1b[6;30;42m',
        'post-run': '\x1b[6;30;44m',
        'failure': '\x1b[6;30;41m'}
    result = job.list_jobstates(context, id=id, sort='created_at')
    jobstates = result.json()['jobstates']

    for js in jobstates:
        color = colors.get(js['status'], '')
        click.echo('%s[%s]\x1b[0m %s' % (
            color,
            js['status'],
            js['comment']))
        f_l = dci_file.list(
            context,
            where='jobstate_id:' + js['id'],
            sort='created_at')
        for f in f_l.json()['files']:
            click.echo(dci_file.content(context, id=f['id']).text)


@cli.command("job-list-test", help="List all tests attached to a job.")
@click.argument("id")
@click.pass_obj
def list_tests(context, id):
    """list_tests(context, id)

    List all tests attached to a job.

    >>> dcictl job-list-test [OPTIONS]

    :param string id: ID of the job [required]
    """

    result = job.list_tests(context, id=id)
    utils.format_output(result, context.format)


@cli.command("job-set-meta", help="Attach an meta to a job.")
@click.argument("id", required=True)
@click.argument("name", required=True)
@click.argument("value", required=False, default='')
@click.pass_obj
def set_meta(context, id, name, value):
    """set_meta(context, id, name, value)

    Attach an meta to a job.

    >>> dcictl job-set-meta [OPTIONS]

    :param string id: ID of the job to attach the meta to [required]
    :param string url: URL of the meta to attach to the job [required]
    :param string name: Key of the meta to attach to the job [required]
    :param string value: Value of the meta to attach to the job
    """

    result = job.set_meta(context, id=id, name=name, value=value)
    utils.format_output(result, context.format)


@cli.command("job-delete-meta", help="Drop a meta from a job.")
@click.argument("id")
@click.argument("meta_id", required=True)
@click.pass_obj
def delete_meta(context, id, meta_id):
    """delete_meta(context, id, meta_id)

    Delete an meta from a job.

    >>> dcictl job-delete-meta [OPTIONS]

    :param string id: ID of the job to attach the meta to [required]
    :param string meta_id: ID of the meta to unattach from the job [required]
    """

    result = job.delete_meta(context, id=id, meta_id=meta_id)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Meta unattached.'})
    else:
        utils.format_output(result, context.format)


@cli.command("job-list-meta", help="List all job attached metas.")
@click.argument("id")
@click.option("--sort", default="-created_at")
@click.option("--limit", default=50)
@click.pass_obj
def list_metas(context, id, sort, limit):
    """list_metas(context, id)

    List all job attached metas.

    >>> dcictl job-list-meta [OPTIONS]

    :param string id: ID of the job to retrieve metas from [required]
    :param string sort: Field to apply sort
    :param integer limit: Max number of rows to return
    """

    result = job.list_metas(context, id=id, sort=sort, limit=limit)
    utils.format_output(result, context.format)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import jobdefinition


@cli.command("jobdefinition-list", help="List all jobdefinitions.")
@click.option("--topic_id")
@click.pass_obj
def list(context, topic_id):
    """list(context, id)

    List all jobdefinitions.

    >>> dcictl jobdefinition-list [OPTIONS]

    :param string topic_id: Topic ID
    """
    result = jobdefinition.list(context, topic_id)
    utils.format_output(result, context.format)


@cli.command("jobdefinition-create", help="Create a jobdefinition.")
@click.option("--name", required=True)
@click.option("--topic_id", required=True)
@click.option("--priority")
@click.option("--component_types")
@click.pass_obj
def create(context, name, topic_id, priority, component_types):
    """create(context, name, priority, topic_id)

    Create a jobdefinition.

    >>> dcictl jobdefinition-create [OPTIONS]

    :param string name: Name of the jobdefinition [required]
    :param string topic_id: ID of the topic for this jobdefinition [required]
    :param integer priority: Priority for this jobdefinition
    :param string component_type: List of component type
    """
    if component_types:
        component_types = component_types.split(',')
    result = jobdefinition.create(context, name=name, priority=priority,
                                  topic_id=topic_id,
                                  component_types=component_types)
    utils.format_output(result, context.format)


@cli.command("jobdefinition-update", help="Update a jobdefinition.")
@click.argument("id")
@click.option("--etag", required=True)
@click.option("--name", required=False)
@click.option("--priority", required=False)
@click.pass_obj
def update(context, id, etag, name, priority):
    """update(context, id, etag, name)

    Update a jobdefinition.

    >>> dcictl jobdefinition-update [OPTIONS]

    :param string id: ID of the jobdefinition to update [required]
    :param string etag: Entity tag of the resource [required]
    :param string name: Name of the jobdefinition
    :param integer priority: Priority of the jobdefinition
    """
    result = jobdefinition.update(context, id=id, etag=etag, name=name,
                                  priority=priority)

    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Job Definition updated.'})
    else:
        utils.format_output(result, context.format)


@cli.command("jobdefinition-delete", help="Delete a jobdefinition.")
@click.argument("id")
@click.option("--etag", required=True)
@click.pass_obj
def delete(context, id, etag):
    """delete(context, id, etag)

    Delete a jobdefinition.

    >>> dcictl jobdefinition-delete [OPTIONS]

    :param string id: ID of the jobdefinition to delete [required]
    :param string etag: Entity tag of the jobdefinition resource [required]
    """
    result = jobdefinition.delete(context, id=id, etag=etag)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Job Definition deleted.'})
    else:
        utils.format_output(result, context.format)


@cli.command("jobdefinition-show", help="Show a jobdefinition.")
@click.argument("id")
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a jobdefinition.

    :param string id: ID of the jobdefinition to show [required]
    """
    result = jobdefinition.get(context, id=id)
    utils.format_output(result, context.format)


@cli.command("jobdefinition-annotate", help="Annotate a jobdefinition.")
@click.argument("id")
@click.option("--comment", required=True)
@click.option("--etag", required=True)
@click.pass_obj
def annotate(context, id, comment, etag):
    """annotate(context, id, comment, etag)

    Annotate a jobdefinition.

    >>> dcictl jobdefinition-annotate [OPTIONS]

    :param string id: ID of the jobdefinition resource [required]
    :param string comment: Comment of the jobdefinition resource [required]
    :param string etag: Entity tag of the jobdefinition resource [required]
    """
    result = jobdefinition.annotate(context, id=id, comment=comment, etag=etag)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Job Definition updated.'})
    else:
        utils.format_output(result, context.format)


@cli.command("jobdefinition-set-active", help="Annotate a jobdefinition.")
@click.argument("id")
@click.option("--active/--no-active", default=True)
@click.option("--etag", required=True)
@click.pass_obj
def setactive(context, id, active, etag):
    """setactive(context, id, active, etag)

    Change the active status of a jobdefinition.

    >>> dcictl jobdefinition-active [OPTIONS]

    :param string id: ID of the jobdefinition resource [required]
    :param boolean active: Active state of the jobdefinition resource
    :param string etag: Entity tag of the jobdefinition resource [required]
    """
    result = jobdefinition.setactive(context, id=id, active=active, etag=etag)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Job Definition updated.'})
    else:
        utils.format_output(result, context.format)


@cli.command("jobdefinition-attach-test",
             help="Attach a test to a jobdefinition.")
@click.argument("id")
@click.option("--test_id", required=True)
@click.pass_obj
def attach_test(context, id, test_id):
    """attach_test(context, id, test_id)

    Attach a test to a jobdefinition.

    >>> dcictl jobdefinition-attach-test [OPTIONS]

    :param string id: ID of the jobdefinition to attach the test to [required]
    :param string test_id: ID of the test to attach [required]
    """
    result = jobdefinition.add_test(context, id=id,
                                    test_id=test_id)
    utils.format_output(result, context.format,
                        ['jobdefinition_id', 'test_id'])


@cli.command("jobdefinition-list-test",
             help="List tests attached to a jobdefinition.")
@click.argument("id")
@click.option("--sort", default="-created_at")
@click.option("--limit", default=50)
@click.pass_obj
def list_test(context, id, sort, limit):
    """list_test(context, id, test_id)

    List tests attached to a jobdefinition.

    >>> dcictl jobdefinition-list-test [OPTIONS]

    :param string id: ID of the jobdefinition to unattach the test from
                      [required]
    :param string sort: Field to apply sort
    :param integer limit: Max number of rows to return
    """
    result = jobdefinition.list_tests(context, id=id, sort=sort, limit=limit)
    utils.format_output(result, context.format)


@cli.command("jobdefinition-unattach-test",
             help="Unattach a test to a jobdefinition.")
@click.argument("id")
@click.option("--test_id", required=True)
@click.pass_obj
def unattach_test(context, id, test_id):
    """unattach_test(context, id, test_id)

    Unattach a test from a jobdefinition.

    >>> dcictl jobdefinition-unattach-test [OPTIONS]

    :param string id: ID of the jobdefinition to unattach the test from
                      [required]
    :param string test_id: ID of the test to unattach [required]
    """
    result = jobdefinition.remove_test(context, id=id,
                                       test_id=test_id)
    if result.status_code == 204:
        unattach_msg = 'Test unattached from Jobdefinition'
        utils.print_json({'id': id,
                          'message': unattach_msg})
    else:
        utils.format_output(result, context.format)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click
import json
import prettytable
import six


def flatten(d, prefix=''):
    ret = []
    for k, v in d.items():
        p = k if not prefix else prefix + '.' + k
        if isinstance(v, dict):
            ret += flatten(v, prefix=p)
        else:
            ret.append("%s=%s" % (p, v))
    return ret


def print_json(result_json):
    formatted_result = json.dumps(result_json, indent=4)
    click.echo(formatted_result)


def _get_field(record, field_path):
    cur_field = field_path.pop(0)
    v = record.get(cur_field)
    if len(field_path):
        return _get_field(record[cur_field], field_path)
    else:
        return v


def print_prettytable(data, headers=None):
    def sort_headers(headers):
        """Ensure the column order is always the same."""
        headers = set(headers)
        sorted_headers = []
        for i in ['id', 'name', 'etag', 'created_at', 'updated_ata']:
            if i not in headers:
                continue
            headers.remove(i)
            sorted_headers.append(i)
        sorted_headers += sorted(headers)
        return sorted_headers

    if isinstance(data, dict):
        keys = [i for i in list(data.keys()) if i != '_meta']
        if len(keys) == 1:
            data = data[keys[0]]

    if not headers:
        if isinstance(data, list):
            first_row = data[0] if len(data) else {}
        else:
            first_row = data
        headers = list(first_row.keys())

    headers = sort_headers(headers)
    table = prettytable.PrettyTable(headers)

    if not isinstance(data, list):
        data = [data]

    for record in data:
        row = []
        for item in headers:
            row.append(_get_field(record, field_path=item.split('/')))
        table.add_row(row)

    click.echo(table)


def sanitize_kwargs(**kwargs):
    boolean_fields = ['active', 'export_control']
    kwargs = dict(
        (k, v) for k, v in six.iteritems(kwargs)
        if (k in boolean_fields and v is not None) or v
    )

    try:
        kwargs['data'] = json.loads(kwargs['data'])
    except KeyError:
        pass
    except TypeError:
        pass

    return kwargs


def format_output(result, format, headers=None,
                  success_code=(200, 201, 204), item=None):

    is_failure = False
    if hasattr(result, 'json'):
        if result.status_code not in success_code:
            is_failure = True
        result = result.json()

    if format == 'json' or is_failure:
        print_json(result)
    else:
        # if our strucutre come with only one root key,
        # we can assume it's the item type.
        if not item and isinstance(result, dict):
            values = list(result.values())
            if len(values) == 1:
                result = values[0]
        to_display = result[item] if item else result
        if to_display:
            print_prettytable(to_display, headers)


def validate_json(ctx, param, value):
    if value is None:
        return
    try:
        return json.loads(value)
    except ValueError:
        raise click.BadParameter('this option expects a valid JSON')



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import jobstate


@cli.command("jobstate-list", help="List all jobstates.")
@click.pass_obj
def list(context):
    """list(context)

    List all jobstates.

    >>> dcictl jobstate-list
    """
    result = jobstate.list(context)
    utils.format_output(result, context.format)


@cli.command("jobstate-show", help="Show a jobstate.")
@click.argument("id")
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a jobstate.

    >>> dcictl jobstate-show [OPTIONS]

    :param string id: ID of the jobstate to show [required]
    """
    result = jobstate.get(context, id=id)
    utils.format_output(result, context.format)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import team
from dciclient.v1.api import test
from dciclient.v1.api import user


@cli.command("test-list", help="List all tests.")
@click.option("--team_id", required=False)
@click.pass_obj
def list(context, team_id):
    """list(context, team_id)

    List all tests.

    >>> dcictl test list

    :param string team_id: ID of the team to list tests [required]
    """
    team_id = team_id or user.list(
        context, where='name:' + context.login).json()['users'][0]['team_id']
    result = team.list_tests(context, team_id)
    utils.format_output(result, context.format)


@cli.command("test-create", help="Create a test.")
@click.option("--name", required=True)
@click.option("--team_id", required=False)
@click.option("--data", callback=utils.validate_json, default='{}')
@click.pass_obj
def create(context, name, team_id, data):
    """create(context, name, team_id, data)

    Create a test.

    >>> dcictl test-create [OPTIONS]

    :param string name: Name of the test [required]
    :param string team_id: ID of the team to associate with
    :param json data: JSON formatted data block for the test
    """
    team_id = team_id or user.list(
        context, where='name:' + context.login).json()['users'][0]['team_id']
    result = test.create(context, name=name, data=data, team_id=team_id)
    utils.format_output(result, context.format)


@cli.command("test-delete", help="Delete a test.")
@click.argument("id")
@click.pass_obj
def delete(context, id):
    """delete(context, id)

    Delete a test.

    >>> dcictl test-delete [OPTIONS]

    :param string id: ID of the test to delete [required]
    """
    result = test.delete(context, id=id)

    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Test deleted.'})
    else:
        utils.format_output(result, context.format)


@cli.command("test-show", help="Show a test.")
@click.argument("id")
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a test.

    >>> dcictl test-show [OPTIONS]

    :param string id: ID of the test to return [required]
    """
    result = test.get(context, id=id)
    utils.format_output(result, context.format)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals


def test_prettytable_output(runner, team_id):
    user = runner.invoke_raw_parse([
        'user-create', '--name', 'foo',
        '--password', 'pass', '--role', 'user',
        '--team_id', team_id])
    assert user['team_id'] == team_id
    assert user == runner.invoke_raw_parse(['user-show', user['id']])


def test_create(runner, team_id):
    user = runner.invoke(['user-create', '--name', 'foo',
                          '--password', 'pass', '--role', 'user',
                          '--team_id', team_id])['user']
    assert user['name'] == 'foo'
    assert user['role'] == 'user'
    assert user['team_id'] == team_id


def test_list(runner, team_id):
    runner.invoke(['user-create', '--name', 'foo',
                   '--password', 'pass', '--role', 'user',
                   '--team_id', team_id])
    runner.invoke(['user-create', '--name', 'bar',
                   '--password', 'pass', '--role', 'user',
                   '--team_id', team_id])
    users = runner.invoke(['user-list'])['users']
    # NOTE (spredzy): We put 5 because of the 3 creates plus
    # admin and 2 users provisionned during server test
    assert len(users) == 5


def test_update(runner, team_id):
    user = runner.invoke(['user-create', '--name', 'foo',
                          '--password', 'pass', '--role', 'user',
                          '--team_id', team_id])['user']

    runner.invoke(['user-update', user['id'],
                   '--etag', user['etag'], '--name', 'bar',
                   '--role', 'admin'])
    user = runner.invoke(['user-show', user['id']])['user']

    assert user['name'] == 'bar'
    assert user['role'] == 'admin'


def test_delete(runner, team_id):
    user = runner.invoke(['user-create', '--name', 'foo',
                          '--password', 'pass', '--role', 'user',
                          '--team_id', team_id])['user']

    result = runner.invoke(['user-delete', user['id'],
                            '--etag', user['etag']])
    assert result['message'] == 'User deleted.'


def test_show(runner, team_id):
    user = runner.invoke(['user-create', '--name', 'foo',
                          '--password', 'pass', '--role', 'user',
                          '--team_id', team_id])['user']

    user = runner.invoke(['user-show', user['id']])['user']

    assert user['name'] == 'foo'



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import os

DEBUG = False

LOG_FILE = '/dev/null'
SQLALCHEMY_DATABASE_URI = os.environ['PIFPAF_POSTGRESQL_URL']
FILES_UPLOAD_FOLDER = '/tmp/dci-control-server'



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import traceback

from dciclient.v1.api import file
from dciclient.v1.api import job
from dciclient.v1.api import jobstate
from dciclient.v1.logger import DciHandler

import json
import logging
from paramiko import ssh_exception
import requests.packages.urllib3

# NOTE(spredzy): The transition to remove completly python-tripleo-helper
# dependency has started. In order for this transition to be has smooth
# has possible we still allow one to rely on it if installed but we don't
# require it for installation.
try:
    import tripleohelper.undercloud
except ImportError:
    raise ImportError('You should install the python-tripleo-helper package')


def push_stack_details(context, undercloud, stack_name='overcloud'):
    undercloud.yum_install(['git'])
    repo_url = 'https://github.com/goneri/tripleo-stack-dump'
    undercloud.run(
        'test -d tripleo-stack-dump || git clone ' + repo_url,
        user='stack')
    undercloud.add_environment_file(
        user='stack',
        filename='stackrc')
    undercloud.run(
        './tripleo-stack-dump/list_nodes_status',
        user='stack')
    undercloud.run(
        './tripleo-stack-dump/tripleo-stack-dump ' + stack_name,
        user='stack')
    with undercloud.open('/home/stack/tripleo-stack-dump.json') as fd:
        j = job.get(
            context,
            id=context.last_job_id).json()['job']
        job.update(
            context,
            id=context.last_job_id,
            etag=j['etag'],
            configuration=json.load(fd))


def run_tests(context, undercloud_ip, key_filename, remoteci_id,
              user='root', stack_name='overcloud'):

    # redirect the log messages to the DCI Control Server
    # https://github.com/shazow/urllib3/issues/523
    requests.packages.urllib3.disable_warnings()
    dci_handler = DciHandler(context)
    logger = logging.getLogger('tripleohelper')
    logger.addHandler(dci_handler)

    undercloud = tripleohelper.undercloud.Undercloud(
        hostname=undercloud_ip,
        user=user,
        key_filename=key_filename)
    undercloud.create_stack_user()

    final_status = 'success'
    if undercloud.run(
            'test -f stackrc',
            user='stack',
            success_status=(0, 1,))[1] != 0:
        msg = 'undercloud deployment failure'
        jobstate.create(context, 'failure', msg, context.last_job_id)
        return
    jobstate.create(
        context,
        'running',
        'Running tripleo-stack-dump',
        context.last_job_id)
    push_stack_details(context, undercloud, stack_name=stack_name)
    rcfile = stack_name + 'rc'
    if undercloud.run(
            'test -f ' + rcfile,
            user='stack',
            success_status=(0, 1,))[1] != 0:
        msg = 'overcloud deployment failure'
        jobstate.create(context, 'failure', msg, context.last_job_id)
        return

    tests = job.list_tests(context, context.last_job_id)
    try:
        for t in tests['tests']:
            if 'url' not in t['data']:
                continue
            jobstate.create(
                context,
                'running',
                'Running test ' + t['name'],
                context.last_job_id)
            url = t['data']['url']
            undercloud.add_environment_file(
                user='stack',
                filename=rcfile)
            undercloud.run('curl -O ' + url, user='stack')
            try:
                undercloud.run((
                    'DCI_REMOTECI_ID=%s '
                    'DCI_JOB_ID=%s '
                    'DCI_OVERCLOUD_STACK_NAME=%s '
                    'bash -x run.sh') % (
                        remoteci_id,
                        context.last_job_id,
                        stack_name), user='stack')
                with undercloud.open('result.xml', user='stack') as fd:
                    file.create(
                        context,
                        t['name'],
                        fd.read(), mime='application/junit',
                        job_id=context.last_job_id)
            except (ssh_exception.SSHException, IOError):
                pass
    except Exception:
        msg = traceback.format_exc()
        final_status = 'failure'
        print(msg)
    else:
        msg = 'test(s) success'
    dci_handler.emit(None)
    jobstate.create(context, final_status, msg, context.last_job_id)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import functools
import io
import logging
import threading

from dciclient.v1.api import file as dci_file


def singleton(cls):
    instances = {}

    def getinstance(handler):
        if cls not in instances:
            instances[cls] = cls(handler)
        return instances[cls]
    return getinstance


@singleton
class DciHandler(logging.Handler):
    def __init__(self, dci_context, interval=30):
        logging.Handler.__init__(self)
        self._dci_context = dci_context
        self._current_log = io.StringIO()
        self._threshold_log = 512 * 1024  # 512K
        self._interval = interval  # seconds
        self._current_jobstate_id = self._dci_context.last_jobstate_id
        self._start_timer()

    def _start_timer(self):
        timer_handle = functools.partial(self.emit, record=None)
        self._timer = threading.Timer(self._interval, timer_handle)
        self._timer.setDaemon(True)
        self._timer.start()

    def _send_log_file(self):
        value = self._current_log.getvalue().encode('UTF-8', 'ignore')
        jobstate_id = self._current_jobstate_id
        if value and jobstate_id:
            r = dci_file.create(self._dci_context, 'logger.txt',
                                value, 'text/plain',
                                jobstate_id)
            if r.status_code != 201:
                raise DciLogPushFailure(r.text)
        self._current_log = io.StringIO()

    def emit(self, record):
        # if record is None then emit() is actually run by the timer
        if record is None:
            self.acquire()
            try:
                if len(self._current_log.getvalue()) > 0:
                    self._send_log_file()
            finally:
                self.release()
            self._start_timer()
            return

        msg = u"%s\n" % self.format(record)
        if self._current_jobstate_id != self._dci_context.last_jobstate_id:
            self._send_log_file()
            self._current_jobstate_id = self._dci_context.last_jobstate_id

        self._current_log.write(msg)
        #  if its an error then send the log
        if record.levelno == logging.ERROR:
            self._send_log_file()
        #  if we reach the current log threshold
        elif len(self._current_log.getvalue()) > self._threshold_log:
            self._send_log_file()


class DciLogPushFailure(Exception):
    """Failed to push log."""



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base
from dciclient.v1.api import jobdefinition
from dciclient.v1.api import remoteci

import json


RESOURCE = 'jobs'


def create(context, recheck, remoteci_id, team_id, jobdefinition_id=None,
           components=None, comment=None):
    job = base.create(context, RESOURCE, recheck=recheck,
                      remoteci_id=remoteci_id, team_id=team_id,
                      jobdefinition_id=jobdefinition_id,
                      components=components,
                      comment=comment)
    context.last_job_id = job.json()['job']['id']
    return job


def update(context, **kwargs):
    return base.update(context, RESOURCE, **kwargs)


def list(context, **kwargs):
    return base.list(context, RESOURCE, **kwargs)


def schedule(context, remoteci_id, topic_id):
    uri = '%s/%s/schedule' % (context.dci_cs_api, RESOURCE)
    data_json = json.dumps({'remoteci_id': remoteci_id, 'topic_id': topic_id})
    r = context.session.post(uri, data=data_json)
    if r.status_code == 201:
        context.last_job_id = r.json()['job']['id']
    return r


def recheck(context, id):
    uri = '%s/%s/%s/recheck' % (context.dci_cs_api, RESOURCE, id)
    r = context.session.post(uri)
    if r.status_code == 201:
        context.last_job_id = r.json()['job']['id']
    return r


def upgrade(context, job_id):
    uri = '%s/%s/upgrade' % (context.dci_cs_api, RESOURCE)
    data_json = json.dumps({'job_id': job_id})
    r = context.session.post(uri, data=data_json)
    if r.status_code == 201:
        context.last_job_id = r.json()['job']['id']
    return r


def get(context, id, **kwargs):
    return base.get(context, RESOURCE, id=id, **kwargs)


def list_results(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id,
                     subresource='results', **kwargs)


def get_full_data(context, id):
    # Get the job with embed on test and remoteci
    embed = ('jobdefinition,remoteci,remoteci.tests,'
             'components,jobdefinition.tests')
    job = base.get(context, RESOURCE, id=id, embed=embed).json()['job']
    return job


def get_components(context, id):
    uri = '%s/%s/%s/components' % (context.dci_cs_api, RESOURCE, id)
    return context.session.get(uri)


def delete(context, id, etag):
    return base.delete(context, RESOURCE, id=id, etag=etag)


def list_issues(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id,
                     subresource='issues', **kwargs)


def attach_issue(context, id, url):
    uri = '%s/%s/%s/issues' % (context.dci_cs_api, RESOURCE, id)
    data_json = json.dumps({'url': url})
    return context.session.post(uri, data=data_json)


def unattach_issue(context, id, issue_id):
    return base.delete(context, RESOURCE, id,
                       subresource='issues',
                       subresource_id=issue_id)


def list_jobstates(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id,
                     subresource='jobstates', **kwargs)


def list_tests(context, id):
    j = base.get(context, RESOURCE, id=id).json()['job']
    result = {'tests': []}
    result['tests'] += jobdefinition.list_tests(
        context, j['jobdefinition_id']).json()['tests']
    result['tests'] += remoteci.list_tests(
        context, j['remoteci_id']).json()['tests']
    return result


def list_metas(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id,
                     subresource='metas', **kwargs)


def set_meta(context, id, name, value):
    uri = '%s/%s/%s/metas' % (context.dci_cs_api, RESOURCE, id)
    data_json = json.dumps({'name': name, 'value': value})
    return context.session.post(uri, data=data_json)


def get_metas(context, id, data):
    uri = '%s/%s/%s/metas' % (context.dci_cs_api, RESOURCE, id)
    return context.session.get(uri)


def delete_meta(context, id, meta_id):
    return base.delete(context, RESOURCE, id,
                       subresource='metas',
                       subresource_id=meta_id)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import dci
import dci.app
import dci.common.utils as dci_utils
import dci.dci_config

import dciclient.shell as shell
import dciclient.v1 as dci_client
import dciclient.v1.api as api
import dciclient.v1.tests.shell_commands.utils as utils

import pytest
import sqlalchemy
import sqlalchemy_utils.functions


import click.testing
import functools
import json
import os
import passlib.apps as passlib_apps


class Mocked_store_engine(object):
    files = {}

    def __init__(self, conf):
        self.container = conf

    def delete(self, filename):
        del(self.files[filename])

    def get(self, filename):
        with open('/tmp/swift/' + filename, 'r') as fd:
            return [None, fd.read()]

    def get_object(self, filename):
        return self.get(filename)[1]

    def head(self, filename):
        return self.files[filename]

    def upload(self, filename, iterable, pseudo_folder=None,
               create_container=True):
        file_path = '/tmp/swift/' + filename
        if not os.path.isdir(os.path.dirname(file_path)):
            os.makedirs(os.path.dirname(file_path))
        with open(file_path, 'wb') as fd:
            if hasattr(iterable, "read"):
                fd.write(iterable.read())
            else:
                fd.write(iterable)
        self.files[filename] = {
            'etag': 'boby',
            'content-type': 'application/octet-stream',
            'content-length': os.stat(file_path).st_size}

    def build_file_path(self, root, middle, file_id):
        root = str(root)
        middle = str(middle)
        file_id = str(file_id)
        return "%s/%s/%s" % (root, middle, file_id)


@pytest.fixture(scope='session')
def engine(request):
    def mocked_get_store(conf):
        store = Mocked_store_engine(conf)
        return store

    dci.dci_config.get_store = mocked_get_store
    conf = dci.dci_config.generate_conf()
    db_uri = conf['SQLALCHEMY_DATABASE_URI']

    engine = sqlalchemy.create_engine(db_uri)

    def del_db():
        if sqlalchemy_utils.functions.database_exists(db_uri):
            sqlalchemy_utils.functions.drop_database(db_uri)

    del_db()
    request.addfinalizer(del_db)
    sqlalchemy_utils.functions.create_database(db_uri)

    dci.db.models.metadata.create_all(engine)
    return engine


@pytest.fixture
def db_clean(request, engine):
    def fin():
        for table in reversed(dci.db.models.metadata.sorted_tables):
            engine.execute(table.delete())
    request.addfinalizer(fin)


@pytest.fixture(scope='session', autouse=True)
def memoize_password_hash():
    pwd_context = passlib_apps.custom_app_context
    pwd_context.verify = dci_utils.memoized(pwd_context.verify)
    pwd_context.encrypt = dci_utils.memoized(pwd_context.encrypt)


@pytest.fixture
def db_provisioning(db_clean, engine):
    with engine.begin() as conn:
        utils.provision(conn)


@pytest.fixture
def server(db_provisioning, engine):
    app = dci.app.create_app(dci.dci_config.generate_conf())
    app.testing = True
    app.engine = engine
    return app


@pytest.fixture
def client(server, db_provisioning):
    client = dci_client.DCIClient(
        end_point='http://dciserver.com/api',
        login='admin', password='admin'
    )
    flask_adapter = utils.FlaskHTTPAdapter(server.test_client())
    client.s.mount('http://dciserver.com', flask_adapter)
    return client


def context_factory(server, db_provisioning, login, password,
                    url='http://dciserver.com', user_agent=None):
    extras = {}
    if user_agent:
        extras['user_agent'] = user_agent
    test_context = api.context.DciContext(url, login, password, **extras)
    flask_adapter = utils.FlaskHTTPAdapter(server.test_client())
    test_context.session.mount(url, flask_adapter)
    return test_context


@pytest.fixture
def dci_context(server, db_provisioning):
    return context_factory(server, db_provisioning, 'admin', 'admin')


@pytest.fixture
def dci_context_user_admin(server, db_provisioning):
    return context_factory(server, db_provisioning, 'user_admin', 'user_admin')


@pytest.fixture
def dci_context_user(server, db_provisioning):
    return context_factory(server, db_provisioning, 'user', 'user')


@pytest.fixture
def dci_context_other_user_agent(server, db_provisioning):
    return context_factory(server, db_provisioning, 'admin', 'admin',
                           user_agent='myagent-0.1')


@pytest.fixture
def dci_context_broken(server, db_provisioning):
    test_context = api.context.DciContext('http://no_where.com',
                                          'admin', 'admin')
    test_context.last_jobstate_id = 1
    test_context.last_job_id = 1
    return test_context


def runner_factory(dci_context):
    api.context.build_dci_context = lambda **kwargs: dci_context
    runner = click.testing.CliRunner(env={'DCI_LOGIN': '', 'DCI_PASSWORD': '',
                                          'DCI_CLI_OUTPUT_FORMAT': 'json'})
    invoke_raw = functools.partial(runner.invoke, shell.main)

    def invoke_json(*kargs):
        r = invoke_raw(*kargs)
        try:
            return json.loads(r.output)
        except ValueError as e:
            print('Failed to JSON decode: >>%s<<' % r.output)
            print('Exit code was: %d' % r.exit_code)
            raise e

    def invoke_raw_parse(*kargs):
        r = invoke_raw(['--format', 'table'] + kargs[0])
        if r.exit_code != 0:
            return r.output
        fields = r.output.split('\n')[1].split('|')
        # NOTE(Gonéri): we only return the very first row because
        # there is no way to know if it a multi-line result
        data = r.output.split('\n')[3].split('|')
        fields = [i.lstrip(' ').rstrip(' ') for i in fields[1:-1]]
        data = [i.lstrip(' ').rstrip(' ') for i in data[1:]]
        result = {}
        for f in fields:
            result[f] = data.pop(0)
        return result

    runner.invoke = invoke_json
    runner.invoke_raw = invoke_raw
    runner.invoke_raw_parse = invoke_raw_parse
    return runner


@pytest.fixture
def runner(dci_context):
    return runner_factory(dci_context)


@pytest.fixture
def runner_user_admin(dci_context_user_admin):
    return runner_factory(dci_context_user_admin)


@pytest.fixture
def runner_user(dci_context_user):
    return runner_factory(dci_context_user)


@pytest.fixture
def team_id(dci_context):
    return api.team.create(dci_context, name='tname').json()['team']['id']


@pytest.fixture
def topic_id(dci_context):
    kwargs = {'name': 'foo_topic'}
    return api.topic.create(dci_context, **kwargs).json()['topic']['id']


@pytest.fixture
def test_id(dci_context, team_id):
    kwargs = {'name': 'test_name', 'team_id': team_id}
    return api.test.create(dci_context, **kwargs).json()['test']['id']


@pytest.fixture
def components(dci_context, topic_id):
    component1 = {'name': 'component1',
                  'type': 'git_commit',
                  'data': {},
                  'canonical_project_name': 'component 1',
                  'topic_id': topic_id}

    component2 = {'name': 'component2',
                  'type': 'git_commit',
                  'data': {},
                  'canonical_project_name': 'component 2',
                  'topic_id': topic_id}

    return [component1, component2]


@pytest.fixture
def remoteci_id(dci_context, team_id):
    kwargs = {'name': 'remoteci',
              'team_id': team_id,
              'data': {'remoteci': 'remoteci'}}
    rci = api.remoteci.create(dci_context, **kwargs).json()
    return rci['remoteci']['id']


@pytest.fixture
def component_id(dci_context, topic_id):
    kwargs = {'name': 'component1', 'type': 'git_review',
              'data': {'component': 'component'}, 'topic_id': topic_id}

    component = api.component.create(dci_context, **kwargs).json()
    return component['component']['id']


@pytest.fixture
def job_factory(dci_context, team_id, topic_id, remoteci_id):
    def create():
        job = api.job.schedule(dci_context, remoteci_id, topic_id).json()
        job_id = job['job']['id']
        api.file.create(dci_context, name='res_junit.xml',
                        content=JUNIT, mime='application/junit',
                        job_id=job_id)
        jobstate_id = api.jobstate.create(
            dci_context, 'pre-run', 'starting',
            job_id).json()['jobstate']['id']
        api.file.create(dci_context, name='pre-run',
                        content='pre-run ongoing', mime='plain/text',
                        jobstate_id=jobstate_id)
        jobstate_id = api.jobstate.create(
            dci_context, 'running', 'starting the build',
            job_id).json()['jobstate']['id']
        return job

    JUNIT = """
    <testsuite errors="0" failures="0" name="pytest" skips="1"
               tests="3" time="46.050">
    <properties>
      <property name="x" value="y" />
      <property name="a" value="b" />
    </properties>
    <testcase classname="" file="test-requirements.txt"
              name="test-requirements.txt" time="0.0109479427338">
        <skipped message="all tests skipped by +SKIP option"
                 type="pytest.skip">Skipped for whatever reasons</skipped>
    </testcase>
    <testcase classname="tests.test_app" file="tests/test_app.py" line="26"
              name="test_cors_preflight" time="2.91562318802"/>
    <testcase classname="tests.test_app" file="tests/test_app.py" line="42"
              name="test_cors_headers" time="0.574683904648"/>
    </testsuite>"""

    kwargs = {'name': 'hihi', 'type': 'type_1', 'topic_id': topic_id,
              'data': {'component': 'component1'}}
    api.component.create(dci_context, **kwargs).json()

    kwargs = {'name': 'haha', 'type': 'type_2', 'topic_id': topic_id,
              'data': {'component': 'component2'}}
    api.component.create(dci_context, **kwargs).json()

    kwargs = {'name': 'tname', 'topic_id': topic_id,
              'component_types': ['type_1', 'type_2']}
    api.jobdefinition.create(dci_context, **kwargs).json()

    api.topic.attach_team(dci_context, topic_id, team_id)
    return create


@pytest.fixture
def job_id(job_factory):
    return job_factory()['job']['id']


@pytest.fixture
def file_id(dci_context, job_id):
    return api.file.list(dci_context).json()['files'][-1]['id']


@pytest.fixture
def jobstate_id(dci_context, job_id):
    kwargs = {'job_id': job_id, 'status': 'running', 'comment': 'some comment'}
    jobstate = api.jobstate.create(dci_context, **kwargs).json()
    return jobstate['jobstate']['id']



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals

from dciclient.v1.api import remoteci
from dciclient.v1.api import team


def test_prettytable_output(runner):
    team = runner.invoke_raw_parse([
        'team-create',
        '--name', 'foo'])
    assert team['name'] == 'foo'
    assert team == runner.invoke_raw_parse([
        'team-show', team['id']])


def test_list(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']
    runner.invoke(['remoteci-create', '--name', 'foo', '--team_id',
                   team['id']])
    runner.invoke(['remoteci-create', '--name', 'bar', '--team_id',
                   team['id']])
    remotecis = runner.invoke(['remoteci-list'])['remotecis']

    assert len(remotecis) == 2
    assert remotecis[0]['name'] == 'bar'
    assert remotecis[1]['name'] == 'foo'


def test_create(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']
    remoteci = runner.invoke([
        'remoteci-create', '--name', 'foo', '--team_id',
        team['id'], '--active'])['remoteci']
    assert remoteci['name'] == 'foo'
    assert remoteci['state'] == 'active'


def test_update(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']
    remoteci = runner.invoke([
        'remoteci-create', '--name', 'foo', '--team_id',
        team['id']])['remoteci']
    result = runner.invoke(['remoteci-update', remoteci['id'],
                            '--etag', remoteci['etag'], '--name', 'bar',
                            '--no-active'])

    assert result['message'] == 'Remote CI updated.'
    assert result['id'] == remoteci['id']


def test_delete(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']

    remoteci = runner.invoke([
        'remoteci-create', '--name', 'foo', '--team_id',
        team['id']])['remoteci']

    result = runner.invoke(['remoteci-delete', remoteci['id'],
                            '--etag', remoteci['etag']])

    assert result['message'] == 'Remote CI deleted.'


def test_show(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']
    remoteci = runner.invoke([
        'remoteci-create', '--name', 'foo', '--team_id',
        team['id']])['remoteci']

    remoteci = runner.invoke(['remoteci-show', remoteci['id']])['remoteci']

    assert remoteci['name'] == 'foo'


def test_get_data(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']
    remoteci = runner.invoke([
        'remoteci-create', '--name', 'foo', '--team_id',
        team['id']])['remoteci']

    result = runner.invoke(['remoteci-get-data', remoteci['id']])
    assert result == {}
    runner.invoke(['remoteci-update', remoteci['id'],
                   '--etag', remoteci['etag'], '--data', '{"foo": "bar"}'])
    result = runner.invoke(['remoteci-get-data', remoteci['id']])
    assert result == {'foo': 'bar'}


def test_get_data_missing_key(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']
    remoteci = runner.invoke([
        'remoteci-create', '--name', 'foo', '--team_id',
        team['id']])['remoteci']

    result = runner.invoke(['remoteci-get-data',
                            remoteci['id'],
                            '--keys', 'missing'])
    assert result == {}


def test_embed(dci_context):
    team_id = team.create(dci_context, name='teama').json()['team']['id']

    rci = remoteci.create(dci_context, name='boa', team_id=team_id).json()
    rci_id = rci['remoteci']['id']

    rci_with_embed = remoteci.get(dci_context,
                                  id=rci_id, embed='team').json()
    embed_team_id = rci_with_embed['remoteci']['team']['id']

    assert team_id == embed_team_id


def test_test(runner, test_id):
    team = runner.invoke(['team-create', '--name', 'osp'])['team']
    remoteci = runner.invoke([
        'remoteci-create', '--name', 'foo', '--team_id',
        team['id'], '--active'])['remoteci']
    result = runner.invoke(['remoteci-attach-test',
                            remoteci['id'], '--test_id', test_id])
    tests = runner.invoke(['remoteci-list-test',
                           remoteci['id']])['tests']
    assert len(tests) == 1
    result = runner.invoke(['remoteci-unattach-test',
                            remoteci['id'], '--test_id', test_id])
    print(result)
    tests = runner.invoke(['remoteci-list-test',
                           remoteci['id']])['tests']
    assert len(tests) == 0



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import job
from dciclient.v1.api import jobdefinition
from dciclient.v1.api import remoteci
from dciclient.v1.api import test

import pytest
import requests
import requests.exceptions


try:
    requests.get('http://google.com')
except requests.exceptions.ConnectionError:
    internet_cnx = False
else:
    internet_cnx = True


def test_get_full_data(job_id, dci_context):
    full_data_job = job.get_full_data(dci_context, job_id)
    assert full_data_job['remoteci']['data'] == {'remoteci': 'remoteci'}
    assert full_data_job['jobdefinition']['name'] == 'tname'
    cpt_names = set([i['name'] for i in full_data_job['components']])
    assert cpt_names == set(['hihi', 'haha'])


def test_list(runner, dci_context, remoteci_id):
    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']

    teams = runner.invoke(['team-list'])['teams']
    team_id = teams[0]['id']

    runner.invoke(['topic-attach-team', topic['id'], '--team_id', team_id])

    jd = runner.invoke(['jobdefinition-create',
                        '--name', 'foo',
                        '--topic_id', topic['id'],
                        '--component_types', 'foobar'])['jobdefinition']

    runner.invoke(['component-create', '--name', 'foo',
                   '--type', 'foobar', '--topic_id',
                   topic['id']])['component']

    job.schedule(dci_context, remoteci_id, topic['id'])
    l_job = runner.invoke(['job-list'])
    assert len(l_job['jobs']) == 1
    assert l_job['jobs'][0]['remoteci']['id'] == remoteci_id
    assert l_job['jobs'][0]['jobdefinition']['id'] == jd['id']
    output = runner.invoke_raw_parse(['job-list'])
    assert output['jobdefinition/name'] == 'foo'
    assert output['id'] == l_job['jobs'][0]['id']

    l_job = runner.invoke(['job-list', '--where',
                           'remoteci_id:' + remoteci_id])
    assert len(l_job['jobs']) == 1


def test_list_with_limit(runner, job_factory):
    for _ in range(6):
        job_factory()
    # test --limit XX
    l_job = runner.invoke(['job-list'])
    assert len(l_job['jobs']) == 6
    l_job = runner.invoke(['job-list', '--limit', 1])
    assert len(l_job['jobs']) == 1


def test_delete(runner, job_id):
    l_job = runner.invoke(['job-show', job_id])
    l_job_etag = l_job['job']['etag']

    result = runner.invoke(['job-delete', job_id,
                            '--etag', l_job_etag])

    assert result['message'] == 'Job deleted.'


def test_recheck(runner, job_id):
    result = runner.invoke(['job-recheck', job_id])['job']

    assert result['status'] == 'new'


def test_results(runner, job_id):
    result = runner.invoke(['job-results', job_id])['results'][0]

    assert result['filename'] == 'res_junit.xml'


@pytest.mark.skipif(not internet_cnx, reason="internet connection required")
def test_attach_issue(runner, job_id):
    result = runner.invoke(['job-list-issue', job_id])['_meta']['count']
    assert result == 0

    issue = runner.invoke(
        ['job-attach-issue', job_id, '--url',
         'https://github.com/redhat-cip/dci-control-server/issues/2']
    )
    # NOTE(Goneri): until we fix the consistency issue with this endpoint:
    # https://softwarefactory-project.io/r/6863
    if 'issue' in issue:
        issue = issue['issue']
    else:
        issue['id'] = issue['issue_id']
    result = runner.invoke(['job-list-issue', job_id])
    assert issue['id'] == result['issues'][0]['id']


@pytest.mark.skipif(not internet_cnx, reason="internet connection required")
def test_unattach_issue(runner, job_id):
    result = runner.invoke(['job-list-issue', job_id])['_meta']['count']
    assert result == 0

    runner.invoke(
        ['job-attach-issue', job_id, '--url',
         'https://github.com/redhat-cip/dci-control-server/issues/2']
    )
    result = runner.invoke(['job-list-issue', job_id])
    res = result['_meta']['count']
    issue_id = result['issues'][0]['id']
    assert res == 1

    runner.invoke(
        ['job-unattach-issue', job_id, '--issue_id', issue_id]
    )
    result = runner.invoke(['job-list-issue', job_id])
    count = result['_meta']['count']
    assert count == 0


def test_job_output(runner, job_id):
    result = runner.invoke_raw(['job-output', job_id])
    assert result.output.startswith('[pre-run]')


def test_job_list(runner, dci_context, team_id, topic_id,
                  remoteci_id, component_id):
    kwargs = {'name': 'tname', 'topic_id': topic_id,
              'component_types': ['git_review']}
    jd = jobdefinition.create(dci_context, **kwargs).json()
    jobdefinition_id = jd['jobdefinition']['id']

    kwargs = {'name': 'test_jobdefinition', 'team_id': team_id}
    test_id = test.create(dci_context, **kwargs).json()['test']['id']
    jobdefinition.add_test(dci_context, jobdefinition_id, test_id)
    kwargs = {'name': 'test_remoteci', 'team_id': team_id}
    test_id = test.create(dci_context, **kwargs).json()['test']['id']
    remoteci.add_test(dci_context, remoteci_id, test_id)

    job_id = job.schedule(
        dci_context, remoteci_id, topic_id).json()['job']['id']
    result = runner.invoke(['job-list-test', job_id])
    assert len(result['tests']) == 2
    assert result['tests'][0]['name'] == 'test_jobdefinition'
    assert result['tests'][1]['name'] == 'test_remoteci'


def test_metas(runner, job_id):
    result = runner.invoke(['job-list-issue', job_id])['_meta']['count']
    assert result == 0
    meta = runner.invoke(['job-set-meta', job_id, 'foo', 'var'])['meta']
    metas = runner.invoke(['job-list-meta', job_id])['metas']
    assert len(metas) == 1
    assert metas[0]['id'] == meta['id']
    assert metas[0]['value'] == 'var'
    runner.invoke(['job-delete-meta', job_id, meta['id']])
    metas = runner.invoke(['job-list-meta', job_id])['metas']
    assert len(metas) == 0



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals


def test_show(runner, file_id):
    result = runner.invoke_raw(['file-show', file_id])
    assert 'testsuite errors' in result.output


def test_list(runner, job_id):
    files = runner.invoke(['file-list', '--job-id', job_id])['files']
    assert len(files)
    assert 'res_junit.xml' in [i['name'] for i in files]


def test_delete(runner, file_id):
    result = runner.invoke(['file-delete', file_id])
    assert result['message'] == 'File deleted.'


def test_list_without_job_id(runner):
    result = runner.invoke_raw(['file-list'])
    assert 'Error: Missing option "--job-id".' in result.output



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals


def test_prettytable_output(runner):
    team = runner.invoke_raw_parse(['team-create', '--name', 'osp'])
    assert team['name'] == 'osp'
    assert team == runner.invoke_raw_parse([
        'team-show', team['id']])


def test_list(runner):
    team = runner.invoke(['team-create', '--name', 'osp'])['team']
    assert team['name'] == 'osp'

    teams = runner.invoke(['team-list'])['teams']
    team_id = teams[0]['id']

    runner.invoke(['test-create', '--name', 'foo', '--team_id', team_id])
    runner.invoke(['test-create', '--name', 'bar', '--team_id', team_id])
    tests = runner.invoke(['test-list', '--team_id', team_id])['tests']
    assert len(tests) == 2
    assert tests[0]['name'] == 'bar'
    assert tests[1]['name'] == 'foo'
    output = runner.invoke_raw_parse(['test-list', '--team_id', team_id])
    assert output['team_id'] == team_id


def test_create(runner):
    team = runner.invoke(['team-create', '--name', 'osp'])['team']
    assert team['name'] == 'osp'

    test = runner.invoke([
        'test-create', '--name', 'foo', '--team_id',
        team['id']])['test']
    assert test['name'] == 'foo'


def test_create_data(runner):
    team = runner.invoke(['team-create', '--name', 'osp'])['team']
    assert team['name'] == 'osp'

    test = runner.invoke([
        'test-create',
        '--name', 'foo',
        '--team_id', team['id'],
        '--data', '{"Foo": 2}'])['test']
    assert test['name'] == 'foo'


def test_create_bad_data(runner):
    team = runner.invoke(['team-create', '--name', 'osp'])['team']
    assert team['name'] == 'osp'

    r = runner.invoke_raw_parse([
        'test-create', 'foo',
        '--team_id', team['id'],
        '--data', '{Foo: 2}'])
    # TODO(Gonéri): should instead ensure we print the fine message
    assert '"--data": this option expects a valid JSON' in r


def test_delete(runner):
    team = runner.invoke(['team-create', '--name', 'osp'])['team']
    assert team['name'] == 'osp'

    test = runner.invoke(['test-create', '--name', 'foo', '--team_id',
                          team['id']])['test']

    result = runner.invoke(['test-delete', test['id']])

    assert result['message'] == 'Test deleted.'


def test_show(runner):
    team = runner.invoke(['team-create', '--name', 'osp'])['team']
    assert team['name'] == 'osp'

    test = runner.invoke(['test-create', '--name', 'foo', '--team_id',
                          team['id']])['test']

    test = runner.invoke(['test-show', test['id']])['test']

    assert test['name'] == 'foo'



# -*- encoding: utf-8 -*-
#
# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base


RESOURCE = 'topics'


def create(context, name, label=None):
    return base.create(context, RESOURCE, name=name, label=label)


def list(context, **kwargs):
    return base.list(context, RESOURCE, **kwargs)


def get(context, id, **kwargs):
    return base.get(context, RESOURCE, id=id, **kwargs)


def update(context, id, etag, name=None, label=None, next_topic=None):
    return base.update(context, RESOURCE, id=id, etag=etag, name=name,
                       label=label, next_topic=next_topic)


def delete(context, id):
    return base.delete(context, RESOURCE, id=id)


def attach_team(context, id, team_id):
    uri = '%s/%s/%s/teams' % (context.dci_cs_api, RESOURCE, id)
    return context.session.post(uri, json={'team_id': team_id})


def unattach_team(context, id, team_id):
    return base.delete(context, RESOURCE, id,
                       subresource='teams',
                       subresource_id=team_id)


def list_attached_team(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource='teams', **kwargs)


def list_components(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id,
                     subresource='components', **kwargs)


def get_jobs_from_components(context, id, component_id, **kwargs):
    uri = '%s/%s/%s/components/%s/jobs' % \
          (context.dci_cs_api, RESOURCE, id, component_id)
    return context.session.get(uri, **kwargs)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base


RESOURCE = 'jobdefinitions'


def create(context, name, topic_id, priority=None, active=None, comment=None,
           component_types=None):
    if component_types is None:
        component_types = []
    return base.create(context, RESOURCE, name=name, priority=priority,
                       active=active, comment=comment, topic_id=topic_id,
                       component_types=component_types)


def list(context, topic_id, **kwargs):
    return base.list(context, RESOURCE, topic_id=topic_id, **kwargs)


def get(context, id, **kwargs):
    return base.get(context, RESOURCE, id=id, **kwargs)


def update(context, id, etag, comment=None, active=None, component_types=None,
           name=None, priority=None):
    if active is not None:
        if active:
            state = 'active'
        else:
            state = 'inactive'
    else:
        state = None

    return base.update(context, RESOURCE, id=id, etag=etag, comment=comment,
                       state=state, component_types=component_types, name=name,
                       priority=priority)


def delete(context, id, etag):
    return base.delete(context, RESOURCE, id=id, etag=etag)


def annotate(context, id, comment, etag):
    return base.update(context, RESOURCE, id=id, etag=etag, comment=comment)


def setactive(context, id, active, etag):
    if active:
        state = 'active'
    else:
        state = 'inactive'

    return base.update(context, RESOURCE, id=id, etag=etag, state=state)


def get_components(context, id):
    uri = '%s/%s/%s/components' % (context.dci_cs_api, RESOURCE, id)
    return context.session.get(uri)


def add_test(context, id, test_id):
    uri = '%s/%s/%s/tests' % (context.dci_cs_api, RESOURCE, id)
    return context.session.post(uri, json={'test_id': test_id})


def list_tests(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource='tests', **kwargs)


def remove_test(context, id, test_id):
    return base.delete(context, RESOURCE, id,
                       subresource='tests',
                       subresource_id=test_id)



# -*- coding: utf-8 -*-
#
# Copyright (C) 2015-2016 Red Hat, Inc
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import component
from dciclient.v1.api import file
from dciclient.v1.api import jobdefinition
from dciclient.v1.api import jobstate
from dciclient.v1.api import test

import fcntl
import mimetypes
import os
import subprocess

import six
import sys
import time


def get_test_id(dci_context, name, topic_id):
    print("Use test '%s'" % name)
    test.create(dci_context, name, topic_id)
    return test.get(dci_context, name).json()['test']['id']


def create_jobdefinition(dci_context, components, test_ids,
                         topic_id, jobdef_name=None):
    # If at least one component doesn't exist in the database then a new
    # jobdefinition must be created.
    at_least_one = False
    component_ids = []
    names = []
    for cmpt in components:
        names.append(cmpt['name'])
        created_cmpt = component.create(dci_context, **cmpt)
        if created_cmpt.status_code == 201:
            at_least_one = True
        elif created_cmpt.status_code in [409, 422]:
            created_cmpt = component.get(dci_context, cmpt['name'])
        created_cmpt_name = created_cmpt.json()['component']['name']
        component_ids.append(created_cmpt.json()['component']['id'])

    if at_least_one:
        if jobdef_name is None:
            jobdef_name = created_cmpt_name
        jobdef = jobdefinition.create(dci_context, jobdef_name, topic_id)
        if jobdef.status_code == 201:
            jobdef_id = jobdef.json()['jobdefinition']['id']
            for cmpt_id in component_ids:
                jobdefinition.add_component(dci_context, jobdef_id, cmpt_id)
            for test_id in test_ids:
                jobdefinition.add_test(dci_context, jobdef_id, test_id)
            print("Jobdefinition '%s' created." % jobdef_name)
        else:
            print("Error on jobdefinition creation: '%s'", jobdef.json())
    else:
        print("No jobdefinition created.")


def upload_file(context, path, job_id, mime=None):

    if not mime:
        mime = mimetypes.guess_type(path)[0]

    l_file = file.create(context, name=path, content=open(path, 'r').read(),
                         mime=mime, job_id=job_id)

    return l_file


def run_command(context, cmd, cwd=None, jobstate_id=None, team_id=None,
                shell=False, expected_retcodes=[0]):
    """This function execute a command and send its log which will be
    attached to a jobstate.
    """
    if not jobstate_id:
        jobstate_id = context.last_jobstate_id
    output = six.StringIO()
    print('* Processing command: %s' % cmd)
    if cmd:
        print('* Working directory: %s' % cwd)
    env = os.environ
    env['DCI_JOB_ID'] = context.last_job_id
    env['DCI_JOBSTATE_ID'] = jobstate_id
    pipe_process = subprocess.Popen(cmd, cwd=cwd,
                                    stdout=subprocess.PIPE,
                                    stderr=subprocess.STDOUT,
                                    shell=shell,
                                    env=env)

    fcntl.fcntl(pipe_process.stdout.fileno(), fcntl.F_SETFL, os.O_NONBLOCK)
    name = cmd if shell else u'_'.join(cmd)

    def flush_buffer(output):
        if output.tell() == 0:
            return
        to_send = output.getvalue()
        file.create(
            context, name=name.encode('utf-8'),
            content=to_send.encode('utf-8'),
            mime='text/plain',
            jobstate_id=jobstate_id)
        output.seek(0)
        output.truncate()

    p_status = None
    while p_status is None:
        time.sleep(0.5)
        try:
            p_status = pipe_process.poll()
            pstdout = pipe_process.stdout.read().decode('UTF-8', 'ignore')
        except IOError:
            pass
        else:
            if len(pstdout) > 0:
                output.write(pstdout)
            if output.tell() > 2048:
                flush_buffer(output)

    pipe_process.wait()
    flush_buffer(output)

    if pipe_process.returncode not in expected_retcodes:
        raise DCIExecutionError()


def run_commands(context, cmds, cwd, jobstate_id, job_id, team_id):
    for cmd in cmds:
        try:
            if isinstance(cmd, dict):
                run_command(context, cmd['cmd'], cmd['cwd'], jobstate_id,
                            team_id)
            else:
                run_command(context, cmd, cwd, jobstate_id, team_id)
        except DCIExecutionError:
            error_msg = "Failed on command %s" % cmd
            jobstate.create(context, "failure", error_msg, job_id)
            sys.exit(1)


class DCIExecutionError(Exception):
    """DCI command excution failure."""
    pass



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals


def test_prettytable_output(runner, topic_id):
    jobdefinition = runner.invoke_raw_parse([
        'jobdefinition-create',
        '--name', 'foo',
        '--topic_id', topic_id])
    assert jobdefinition == runner.invoke_raw_parse([
        'jobdefinition-show', jobdefinition['id']])


def test_list(runner, topic_id):
    teams = runner.invoke(['team-list'])['teams']
    team_id = teams[0]['id']

    runner.invoke(['topic-attach-team', topic_id,
                   '--team_id', team_id])

    runner.invoke(['jobdefinition-create', '--name', 'foo', '--topic_id',
                  topic_id])
    runner.invoke(['jobdefinition-create', '--name', 'bar', '--topic_id',
                  topic_id])

    jobdefinitions = runner.invoke([
        'jobdefinition-list',
        '--topic_id', topic_id])['jobdefinitions']

    assert len(jobdefinitions) == 2
    assert jobdefinitions[0]['name'] == 'bar'
    assert jobdefinitions[1]['name'] == 'foo'


def test_create(runner, topic_id):
    jobdefinition = runner.invoke([
        'jobdefinition-create', '--name', 'foo',
        '--topic_id', topic_id])['jobdefinition']
    assert jobdefinition['name'] == 'foo'


def test_delete(runner, topic_id):
    jobdefinition = runner.invoke([
        'jobdefinition-create', '--name', 'foo',
        '--topic_id', topic_id])['jobdefinition']

    result = runner.invoke(['jobdefinition-delete',
                            jobdefinition['id'], '--etag',
                            jobdefinition['etag']])
    assert result['message'] == 'Job Definition deleted.'


def test_update(runner, topic_id):
    jd = jobdefinition = runner.invoke([
        'jobdefinition-create', '--name', 'foo',
        '--topic_id', topic_id])['jobdefinition']

    jobdefinition = runner.invoke([
        'jobdefinition-show', jd['id']])['jobdefinition']
    assert jobdefinition['name'] == 'foo'
    assert jobdefinition['priority'] == 0

    result = runner.invoke([
        'jobdefinition-update', jd['id'],
        '--etag', jd['etag'], '--name', 'bar',
        '--priority', 10])

    assert result['message'] == 'Job Definition updated.'

    jobdefinition = runner.invoke([
        'jobdefinition-show', jd['id']])['jobdefinition']
    assert jobdefinition['name'] == 'bar'
    assert jobdefinition['priority'] == 10


def test_show(runner, topic_id):
    jobdefinition = runner.invoke([
        'jobdefinition-create', '--name', 'foo',
        '--topic_id', topic_id])['jobdefinition']

    jobdefinition = runner.invoke([
        'jobdefinition-show',
        jobdefinition['id']])['jobdefinition']

    assert jobdefinition['name'] == 'foo'


def test_annotate(runner, topic_id):
    jd = runner.invoke([
        'jobdefinition-create', '--name', 'foo',
        '--topic_id', topic_id])['jobdefinition']
    result = runner.invoke(['jobdefinition-annotate', jd['id'],
                            '--comment', 'This is my annotation', '--etag',
                            jd['etag']])
    assert result['message'] == 'Job Definition updated.'

    result = runner.invoke(['jobdefinition-show', jd['id']])
    jobdefinition_comm = result['jobdefinition']['comment']
    assert jobdefinition_comm == 'This is my annotation'


def test_active(runner, topic_id):
    jd = runner.invoke(['jobdefinition-create', '--name', 'foo',
                        '--topic_id', topic_id])['jobdefinition']

    result = runner.invoke(['jobdefinition-set-active', '--no-active',
                            jd['id'], '--etag', jd['etag']])

    assert result['message'] == 'Job Definition updated.'
    jobdefinition_active = runner.invoke([
        'jobdefinition-show',
        jd['id']])['jobdefinition']['state']
    assert jobdefinition_active == 'inactive'


def test_test(runner, topic_id, test_id):
    jd = runner.invoke(['jobdefinition-create', '--name', 'foo',
                        '--topic_id', topic_id])['jobdefinition']

    result = runner.invoke(['jobdefinition-attach-test',
                            jd['id'], '--test_id', test_id])
    tests = runner.invoke(['jobdefinition-list-test',
                           jd['id']])['tests']
    assert len(tests) == 1
    result = runner.invoke(['jobdefinition-unattach-test',
                            jd['id'], '--test_id', test_id])
    print(result)
    tests = runner.invoke(['jobdefinition-list-test',
                           jd['id']])['tests']
    assert len(tests) == 0



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


import dciclient.v1.api.file as dci_file


def test_iter(dci_context, job_id):
    f_names = ['file_%d' % i for i in range(100)]
    for name in f_names:
        dci_file.create(
            dci_context, name=name,
            content='some content', mime='plain/text',
            job_id=job_id)
    cpt = 0
    seen_names = []
    for f in dci_file.iter(dci_context):
        seen_names.append(f['name'])
        cpt += 1
    # job already comes with 2 files
    all_files = len(dci_file.list(dci_context).json()['files'])
    assert all_files == 100 + 2
    assert cpt == 100 + 2
    assert len(set(seen_names) - set(f_names)) == 2



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import base


@cli.command("purge", help="Purge soft-deleted resources.")
@click.option("--resource", help="Comma separated list of resource to purge.")
@click.option("--noop", is_flag=True, help="Show resource to purge.")
@click.pass_obj
def purge(context, resource, noop):
    """purge(context, resource, noop)

    Purge soft-deleted resources.

    >>> dcictl purge --resource remotecis
    """

    resources = ['components', 'topics', 'tests', 'teams', 'jobdefinitions',
                 'remotecis', 'jobs', 'files', 'users']

    l_resources = resources if resource is None else resource.split(',')

    wrong_resources = [res for res in l_resources if res not in resources]
    test_auth = base.purge(context, 'users', **{'noop': True})

    if len(wrong_resources) > 0:
        msg = 'Unkown resource have been specified: %s' % wrong_resources
        if context.format == 'json':
            utils.print_json(msg)
        else:
            click.echo(msg)

    elif test_auth.status_code == 401:
        utils.format_output(test_auth, context.format)

    else:
        purged = {}
        if noop:
            # If in noop mode. The various endpoints are queried for the
            # informations about the resources to be purged and displayed.
            for res in l_resources:
                resource_to_delete = base.purge(context, res, **{'noop': noop})
                if resource_to_delete.json()['_meta']['count'] > 0:
                    purged[res] = resource_to_delete.json()
            if len(purged.keys()):
                for item in purged.keys():
                    if len(l_resources) > 1:
                        click.echo('\n%s:\n' % item)
                    utils.format_output(purged[item][item], context.format)
            else:
                utils.format_output({}, context.format)

        else:
            # If not in noop mode. First we retrieve the number of items to be
            # purged and then we purge them. This allows to presents meaningful
            # informations to the user that used this command.

            for res in l_resources:
                item_purged = base.purge(context, res, **{'noop': True}) \
                                  .json()['_meta']['count']
                if item_purged and \
                   base.purge(context, res,
                              **{'noop': False}).status_code == 204:
                    purged[res] = '%s item(s) purged' % item_purged
            if len(purged.keys()):
                utils.print_json(purged)
            else:
                utils.print_json({'message': 'No item to be purged'})



# -*- encoding: utf-8 -*-
#
# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import component
from dciclient.v1.api import job
from dciclient.v1.api import jobdefinition
from dciclient.v1.api import topic


def test_job_updated(dci_context, job_id):
    new_configuration = {'foo': 'bar'}
    j = job.get(dci_context, job_id).json()['job']
    assert j['configuration'] == {}
    job.update(dci_context, id=job_id, etag=j['etag'],
               configuration=new_configuration)
    r = job.get(dci_context, job_id)
    j = r.json()['job']
    assert j['configuration'] == new_configuration


def test_get_full_data(dci_context, job_id):
    j = job.get_full_data(dci_context, job_id)
    assert j['jobdefinition']['tests'] == []
    assert j['remoteci']['tests'] == []
    assert len(j['components']) > 1


def test_job_upgraded(dci_context, job_id, topic_id):
    old = topic.get(dci_context, id=topic_id).json()['topic']
    new = topic.create(dci_context, 'bar_topic')
    t = new.json()['topic']
    component.create(dci_context, 'bar_component', 'type_1', t['id'])
    jobdefinition.create(dci_context, 'bar_jobdef', t['id'],
                         component_types=['type_1'])
    topic.update(dci_context, id=topic_id, etag=old['etag'],
                 next_topic=t['id'])
    r = job.upgrade(dci_context, job_id=job_id)
    assert r.status_code == 201
    assert r.json()['job']['previous_job_id'] == job_id



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from dciclient.v1.api import base

import json
import os


RESOURCE = 'components'


def create(context, name, type, topic_id, canonical_project_name=None, data={},
           title=None, message=None, url=None, export_control=True):
    return base.create(context, RESOURCE, name=name, type=type,
                       canonical_project_name=canonical_project_name,
                       data=json.dumps(data), title=title, message=message,
                       url=url, topic_id=topic_id,
                       export_control=export_control)


def get(context, id, **kwargs):
    return base.get(context, RESOURCE, id=id, **kwargs)


def update(context, id=None, etag=None, name=None, content=None,
           export_control=None):
    return base.update(context, RESOURCE, id=id, etag=etag, name=name,
                       content=content, export_control=export_control)


def delete(context, id):
    return base.delete(context, RESOURCE, id=id)


def file_upload(context, id, file_path):
    uri = '%s/%s/%s/files' % (context.dci_cs_api, RESOURCE, id)
    with open(file_path, 'rb') as f:
        return context.session.post(uri, data=f)


def file_get(context, id, file_id):
    uri = '%s/%s/%s/files/%s' % (context.dci_cs_api, RESOURCE, id, file_id)
    return context.session.get(uri)


def file_download(context, id, file_id, target):
    uri = '%s/%s/%s/files/%s/content' % (
        context.dci_cs_api, RESOURCE, id, file_id)
    r = context.session.get(uri, stream=True)
    r.raise_for_status()
    with open(target + '.part', 'wb') as f:
        for chunk in r.iter_content(chunk_size=1024):
            if chunk:
                f.write(chunk)
    os.rename(target + '.part', target)


def file_list(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id, subresource='files', **kwargs)


def file_delete(context, id, file_id):
    return base.delete(context, RESOURCE, id,
                       subresource='files',
                       subresource_id=file_id)


def status(context, type, topic_id):
    uri = '%s/%s/%s/type/%s/status' % (context.dci_cs_api, 'topics', topic_id,
                                       type)
    return context.session.get(uri)


def list_issues(context, id, **kwargs):
    return base.list(context, RESOURCE, id=id,
                     subresource='issues', **kwargs)


def attach_issue(context, id, url):
    uri = '%s/%s/%s/issues' % (context.dci_cs_api, RESOURCE, id)
    data_json = json.dumps({'url': url})
    return context.session.post(uri, data=data_json)


def unattach_issue(context, id, issue_id):
    return base.delete(context, RESOURCE, id,
                       subresource='issues',
                       subresource_id=issue_id)



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals


def test_prettytable_output(runner):
    team = runner.invoke_raw_parse([
        'team-create',
        '--name', 'foo'])
    assert team['name'] == 'foo'
    assert team == runner.invoke_raw_parse([
        'team-show', team['id']])


def test_list(runner):
    runner.invoke(['team-create', '--name', 'foo'])
    runner.invoke(['team-create', '--name', 'bar'])
    teams = runner.invoke(['team-list'])['teams']
    # NOTE (spredzy): We put 4 because of the 2 creates plus
    # admin and user provisionned during server test
    assert len(teams) == 4


def test_create(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']
    assert team['name'] == 'foo'


def test_create_fail_unauthorized_user_admin(runner_user_admin):
    team = runner_user_admin.invoke(['team-create', '--name', 'foo'])
    assert team['status_code'] == 401


def test_create_fail_unauthorized_user(runner_user):
    team = runner_user.invoke(['team-create', '--name', 'foo'])
    assert team['status_code'] == 401


def test_update(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']

    result = runner.invoke(['team-update', team['id'],
                            '--etag', team['etag'], '--name', 'bar'])

    assert result['message'] == 'Team updated.'
    assert result['id'] == team['id']


def test_delete(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']

    result = runner.invoke(['team-delete', team['id'],
                            '--etag', team['etag']])

    assert result['message'] == 'Team deleted.'


def test_show(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']

    team = runner.invoke(['team-show', team['id']])['team']

    assert team['name'] == 'foo'



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import click

from dciclient.v1.shell_commands import cli
from dciclient.v1 import utils

from dciclient.v1.api import component
from dciclient.v1.api import topic


@cli.command("component-list", help="List all components.")
@click.option("--topic_id", required=True)
@click.option("--sort", default="-created_at")
@click.option("--limit", default=50)
@click.pass_obj
def list(context, topic_id, sort, limit):
    """list(context, topic_id)

    List all components.

    >>> dcictl component-list [OPTIONS]

    :param string topic_id: The topic ID for the list of components to return
                            [required]
    :param string sort: Field to apply sort
    :param integer limit: Max number of rows to return
    """
    components = topic.list_components(context, id=topic_id,
                                       sort=sort, limit=limit)
    utils.format_output(components, context.format)


@cli.command("component-create", help="Create a component.")
@click.option("--name", required=True, help='Name of component')
@click.option("--type", required=True, help='Type of component')
@click.option("--canonical_project_name", help='Canonical project name')
@click.option("--data", default='{}', callback=utils.validate_json,
              help='Data to pass (in JSON)')
@click.option("--title", help='Title of component')
@click.option("--message", help='Component message')
@click.option("--url", help='URL to look for the component')
@click.option("--topic_id", required=True, help='Topic ID')
@click.option("--export_control/--no-export_control", default='true',
              help='has the export_control been done')
@click.pass_obj
def create(context, name, type, canonical_project_name, data,
           title, message, url, topic_id, export_control):
    """create(context, name, type, canonical_project_name, data, title, message, url, topic_id, export_control)  # noqa

    Create a component.

    >>> dcictl component-create [OPTIONS]

    :param string name: Name of the component [required]
    :param string type: Type of the component [required]
    :param string topic_id: ID of the topic to associate with [required]
    :param string canonical_project_name: Project name
    :param json data: JSON to pass to the component
    :param string title: Title of the component
    :param string message: Message for the component
    :param string url: URL resource to monitor
    :param boolean export_control: Set the component visible for users
    """
    result = component.create(
        context, name=name, type=type,
        canonical_project_name=canonical_project_name,
        data=data,
        title=title, message=message, url=url,
        topic_id=topic_id, export_control=export_control
    )
    utils.format_output(result, context.format)


@cli.command("component-delete", help="Delete a component.")
@click.argument("id")
@click.pass_obj
def delete(context, id):
    """delete(context, id)

    Delete a component.

    >>> dcictl component-delete [OPTIONS]

    :param string id: ID of the component to delete [required]
    """
    result = component.delete(context, id=id)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Component deleted.'})
    else:
        utils.format_output(result, context.format)


@cli.command("component-show", help="Show a component.")
@click.argument("id")
@click.pass_obj
def show(context, id):
    """show(context, id)

    Show a component

    >>> dcictl component-show [OPTIONS]

    :param string id: ID of the component to show [required]
    """
    result = component.get(context, id=id)
    utils.format_output(result, context.format)


@cli.command("component-status",
             help="Show an overview of the last jobs associated to this component type")  # noqa
@click.option("--type", required=True)
@click.option("--topic_id", required=True)
@click.pass_obj
def status(context, type, topic_id):
    """status(context, type, topic_id)

    Show current status of a component

    >>> dcictl component-status [OPTIONS]

    :param string type: Type of component to look up status for [required]
    :param string topic_id: The topic ID for the list of components to return
                            [required]
    """
    result = component.status(context, type=type, topic_id=topic_id)
    utils.format_output(result, context.format)


@cli.command("component-file-upload", help="Attach a file to a component.")
@click.argument("id")
@click.option("--path", required=True)
@click.pass_obj
def file_upload(context, id, path):
    """file_upload(context, id, path)

    Upload a file in a component

    >>> dcictl component-file-upload [OPTIONS]

    :param string id: ID of the component to attach file [required]
    :param string path: Path to the file to upload [required]
    """
    result = component.file_upload(context, id=id, file_path=path)
    utils.format_output(result, context.format)


@cli.command("component-file-show", help="Show a component file.")
@click.argument("id")
@click.option("--file_id", required=True)
@click.pass_obj
def file_show(context, id, file_id):
    """file_show(context, id, path)

    Show a component file

    >>> dcictl component-file-show [OPTIONS]

    :param string id: ID of the component to show files [required]
    :param string file_id: ID of the file to show up [required]
    """
    result = component.file_get(context, id=id, file_id=file_id)
    utils.format_output(result, context.format)


@cli.command("component-file-download", help="Retrieve a component file.")
@click.argument("id")
@click.option("--file_id", required=True)
@click.option("--target", required=True)
@click.pass_obj
def file_download(context, id, file_id, target):
    """file_download(context, id, path)

    Download a component file

    >>> dcictl component-file-download [OPTIONS]

    :param string id: ID of the component to download file [required]
    :param string file_id: ID of the component file to download [required]
    :param string target: Destination file [required]
    """
    component.file_download(context, id=id, file_id=file_id, target=target)


@cli.command("component-file-list", help="List files attached to a component.")
@click.argument("id")
@click.option("--sort", default="-created_at")
@click.option("--limit", default=50)
@click.pass_obj
def file_list(context, id, sort, limit):
    """file_list(context, id, path)

    List component files

    >>> dcictl component-file-list [OPTIONS]

    :param string id: ID of the component to list files [required]
    :param string sort: Field to apply sort
    :param integer limit: Max number of rows to return
    """
    result = component.file_list(context, id=id, sort=sort, limit=limit)
    utils.format_output(result, context.format)


@cli.command("component-file-delete", help="Delete a component file.")
@click.argument("id")
@click.option("--file_id", required=True)
@click.pass_obj
def file_delete(context, id, file_id):
    """file_delete(context, id, path)

    Delete a component file

    >>> dcictl component-file-delete [OPTIONS]

    :param string id: ID of the component to delete file [required]
    :param string file_id: ID for the file to delete [required]
    """
    result = component.file_delete(context, id=id, file_id=file_id)
    utils.format_output(result, context.format)


@cli.command("component-update", help="Update a component.")
@click.argument("id")
@click.option("--export-control/--no-export-control")
@click.pass_obj
def update(context, id, export_control):
    """update(context, id, export_control)

    Update a component

    >>> dcictl component-update [OPTIONS]

    :param string id: ID of the component [required]
    :param boolean export-control: Set the component visible for users
    """

    component_info = component.get(context, id=id)

    etag = component_info.json()['component']['etag']
    result = component.update(context, id=id, etag=etag,
                              export_control=export_control)

    if result.status_code == 204:
        status = 'Enabled' if export_control else 'Disabled'
        utils.print_json({'id': id, 'message': 'Export Control %s.' % status})
    else:
        utils.format_output(result, context.format)


@cli.command("component-attach-issue", help="Attach an issue to a component.")
@click.argument("id")
@click.option("--url", required=True)
@click.pass_obj
def attach_issue(context, id, url):
    """attach_issue(context, id, url)

    Attach an issue to a component.

    >>> dcictl component-attach-issue [OPTIONS]

    :param string id: ID of the component to attach the issue to [required]
    :param string url: URL of the issue to attach to the component[required]
    """

    result = component.attach_issue(context, id=id, url=url)
    if result.status_code == 201:
        utils.print_json({'id': id, 'message': 'Issue attached.'})
    else:
        utils.format_output(result, context.format)


@cli.command("component-unattach-issue",
             help="Unattach an issue from a component.")
@click.argument("id")
@click.option("--issue_id", required=True)
@click.pass_obj
def unattach_issue(context, id, issue_id):
    """unattach_issue(context, id, issue_id)

    Unattach an issue from a component.

    >>> dcictl component-unattach-issue [OPTIONS]

    :param string id: ID of the component to unattach the issue from [required]
    :param string issue_id: ID of the issue to unattach from the component[required]  # noqa
    """

    result = component.unattach_issue(context, id=id, issue_id=issue_id)
    if result.status_code == 204:
        utils.print_json({'id': id, 'message': 'Issue unattached.'})
    else:
        utils.format_output(result, context.format)


@cli.command("component-list-issue",
             help="List all component attached issues.")
@click.argument("id")
@click.option("--sort", default="-created_at")
@click.option("--limit", default=50)
@click.pass_obj
def list_issues(context, id, sort, limit):
    """list_issues(context, id)

    List all component attached issues.

    >>> dcictl component-list-issue [OPTIONS]

    :param string id: ID of the component to retrieve issues from [required]
    """

    result = component.list_issues(context, id=id, sort=sort, limit=limit)
    headers = ['id', 'status', 'product', 'component', 'title', 'url']
    utils.format_output(result, context.format, headers)



# -*- encoding: utf-8 -*-
#
# Copyright 2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals


def test_prettytable_output(runner):
    topic = runner.invoke_raw_parse([
        'topic-create', '--name', 'osp'])
    assert topic == runner.invoke_raw_parse([
        'topic-show', topic['id']])


def test_list(runner):
    runner.invoke(['topic-create', '--name', 'osp'])
    runner.invoke(['topic-create', '--name', 'ovirt'])
    topics = runner.invoke(['topic-list'])['topics']

    assert len(topics) == 2
    # assert topics[0]['name'] == 'ovirt'
    # assert topics[1]['name'] == 'osp'


def test_create(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']
    assert topic['name'] == 'osp'


def test_delete(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']
    result = runner.invoke(['topic-delete', topic['id']])

    assert result['message'] == 'Topic deleted.'


def test_show(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']
    topic = runner.invoke(['topic-show', topic['id']])['topic']

    assert topic['name'] == 'osp'


def test_attach_team(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']
    assert team['name'] == 'foo'

    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']
    assert topic['name'] == 'osp'

    topic_team = runner.invoke(['topic-attach-team', topic['id'],
                                '--team_id', team['id']])
    assert topic_team['team_id'] == team['id']
    assert topic_team['topic_id'] == topic['id']


def test_list_team(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']
    assert team['name'] == 'foo'

    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']
    assert topic['name'] == 'osp'

    topic_team = runner.invoke(['topic-attach-team', topic['id'],
                                '--team_id', team['id']])
    assert topic_team['team_id'] == team['id']
    assert topic_team['topic_id'] == topic['id']

    result = runner.invoke(['topic-list-team', topic['id']])['teams']
    assert len(result) == 1
    assert result[0]['name'] == 'foo'


def test_unattach_team(runner):
    team = runner.invoke(['team-create', '--name', 'foo'])['team']
    assert team['name'] == 'foo'

    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']
    assert topic['name'] == 'osp'

    topic_team = runner.invoke(['topic-attach-team', topic['id'],
                                '--team_id', team['id']])
    assert topic_team['team_id'] == team['id']
    assert topic_team['topic_id'] == topic['id']

    result = runner.invoke(['topic-list-team', topic['id']])['teams']
    assert len(result) == 1
    assert result[0]['name'] == 'foo'

    topic_team = runner.invoke(['topic-unattach-team', topic['id'],
                                '--team_id', team['id']])

    teams = runner.invoke(['topic-list-team', topic['id']])['teams']
    assert len(teams) == 0



# -*- encoding: utf-8 -*-
#
# Copyright 2015-2016 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from __future__ import unicode_literals


def test_prettytable_output(runner, topic_id):
    component = runner.invoke_raw_parse([
        'component-create',
        '--name', 'osp',
        '--type', 'repo',
        '--topic_id', topic_id])
    assert component['name'] == 'osp'
    assert component == runner.invoke_raw_parse([
        'component-show', component['id']])


def test_list(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']
    teams = runner.invoke(['team-list'])['teams']
    team_id = teams[0]['id']

    runner.invoke(['topic-attach-team', topic['id'],
                   '--team_id', team_id])

    runner.invoke(['component-create', '--name', 'foo', '--type', 'bar',
                   '--topic_id', topic['id']])
    runner.invoke(['component-create', '--name', 'bar', '--type', 'bar2',
                   '--topic_id', topic['id']])
    components = runner.invoke([
        'component-list', '--topic_id', topic['id']])['components']

    assert len(components) == 2
    assert components[0]['name'] == 'bar'
    assert components[1]['name'] == 'foo'


def test_create(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']

    component = runner.invoke(['component-create', '--name', 'foo',
                               '--type', 'foobar', '--topic_id',
                               topic['id']])['component']
    assert component['name'] == 'foo'


def test_delete(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']
    component = runner.invoke(['component-create', '--name', 'foo',
                               '--type', 'bar', '--topic_id',
                               topic['id']])['component']

    result = runner.invoke(['component-delete', component['id']])

    assert result['message'] == 'Component deleted.'


def test_show(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']
    assert topic['name'] == 'osp'

    component = runner.invoke([
        'component-create', '--name', 'foo',
        '--type', 'bar', '--export_control',
        '--topic_id', topic['id']])['component']

    result = runner.invoke(['component-show', component['id']])

    assert result['component']['name'] == 'foo'


def test_file_support(runner, tmpdir):
    td = tmpdir
    p = td.join("hello.txt")
    p.write("content")
    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']

    component = runner.invoke(['component-create', '--name', 'foo',
                               '--type', 'foobar', '--topic_id', topic['id'],
                               '--export_control'])['component']

    # upload
    new_f = runner.invoke(['component-file-upload', component['id'],
                           '--path', p.strpath])['component_file']
    assert new_f['size'] == 7

    # show
    new_f = runner.invoke(['component-file-show', component['id'],
                           '--file_id', new_f['id']])['component_file']
    assert new_f['size'] == 7

    # download
    runner.invoke_raw(['component-file-download', component['id'],
                       '--file_id', new_f['id'],
                       '--target', td.strpath + '/my_file'])
    assert open(td.strpath + '/my_file', 'r').read() == 'content'

    # list
    my_list = runner.invoke(['component-file-list',
                             component['id']])['component_files']
    assert len(my_list) == 1
    assert my_list[0]['size'] == 7

    # delete
    runner.invoke_raw([
        'component-file-delete', component['id'],
        '--file_id', new_f['id']])
    result = runner.invoke(['component-file-show', component['id'],
                            '--file_id', new_f['id']])
    assert result['status_code'] == 404


def test_enable_export_control(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']

    component = runner.invoke([
        'component-create', '--name', 'foo',
        '--type', 'bar', '--topic_id',
        topic['id']])['component']

    result = runner.invoke(['component-update', component['id'],
                            '--export-control'])
    assert result['message'] == 'Export Control Enabled.'


def test_disable_export_control(runner):
    topic = runner.invoke(['topic-create', '--name', 'osp'])['topic']

    component = runner.invoke([
        'component-create', '--name', 'foo',
        '--type', 'bar', '--topic_id',
        topic['id']])['component']

    result = runner.invoke(['component-update', component['id'],
                            '--export-control'])
    assert result['message'] == 'Export Control Enabled.'

    result = runner.invoke(['component-update', component['id'],
                            '--no-export-control'])
    assert result['message'] == 'Export Control Disabled.'


def test_component_status(runner, job_id, topic_id):

    status = runner.invoke(['component-status', '--type', 'type_1',
                            '--topic_id', topic_id])['jobs']

    assert len(status) == 0
